#ifndef	GRAPH_H_
#define	GRAPH_H_

#include "list.h"

#define	GRAPH_OK		0
#define	GRAPH_INVALID_ARGS	1
#define	GRAPH_NO_MEMORY		2

typedef struct graph graph;
typedef struct graph_node graph_node;
typedef struct graph_edge graph_edge;

typedef int (*node_cmp)(const void *, const void *);
typedef int (*edge_cmp)(const void *, const void *);


struct graph {
	list *nodes;
};

struct graph_node {
	list *in_edges;
	list *out_edges;

	void *data;
};

struct graph_edge {
	graph_node *src;
	graph_node *dst;

	void *data;
};

/* create a new directed graph structure */
graph *graph_new(void);

/* free all resources occupied by graph, its nodes and edges */
void graph_free(graph **gp);

/* create a new node structure */
graph_node *graph_new_node(void);

/*
 * set node's data;
 * return old value;
 */
void *graph_set_node_data(graph_node *np, void *data);

/* add node to the graph */
int graph_add_node(graph *gp, graph_node *np);

/*
 * look up for a node and return pointer to it;
 * cmp() should compare nodes by their data;
 * cmp() should return non-zero value if nodes are different;
 * if insert is not 0, inserts the node into the graph;
 */
graph_node *graph_lookup_node(graph *gp, graph_node *np, node_cmp cmp, int insert);

/* remove node */
int graph_del_node(graph *gp, graph_node *np);

/* create a new edge structure */
graph_edge *graph_new_edge(void);

/*
 * set edge's data;
 * return old value;
 */
void *graph_set_edge_data(graph_edge *ep, void *data);

/* set edge's source */
graph_node *graph_set_edge_src(graph_edge *ep, graph_node *np);

/* set edge's destination */
graph_node *graph_set_edge_dst(graph_edge *ep, graph_node *np);

/* add new edge */
int graph_add_edge(graph_edge *ep);

/* look up edge and return pointer to it */
graph_edge *graph_lookup_edge(graph_edge *ep, edge_cmp cmp, int insert);

/* remove edge */
void graph_del_edge(graph_edge *ep);

#endif

